package com.laguerre.kan;

import android.opengl.GLES20;

public abstract class Texture {
	private static int textureCounter = 0;

	public static void resetTextureCounter() {
		textureCounter = 0;
	}
	public static int getThenIncrTextureCounter() {
		final int n = textureCounter;
		textureCounter++;
		return n;
	}

	private static int maxCombinedTextures = -1;
	protected final int[] textureHandle = { -1 };

	public abstract int getWidth();

	public abstract int getHeight();

	public abstract void create();

	protected final void doCreation() {
		GLES20.glGenTextures(1, this.textureHandle, 0);

		if (this.textureHandle[0] > 0) {
			// Bind to the texture in OpenGL
			GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, this.textureHandle[0]);

			// Set filtering
			boolean mipmap = false;
			if (mipmap) {
				GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR_MIPMAP_LINEAR);
			} else {
				GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_NEAREST);
			}
			//GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_NEAREST);
			GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_NEAREST);

			GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, GLES20.GL_CLAMP_TO_EDGE);
			GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, GLES20.GL_CLAMP_TO_EDGE);

			if (mipmap) {
				GLES20.glGenerateMipmap(GLES20.GL_TEXTURE_2D);
			}
		} else {
			throw new RuntimeException("Error loading texture.");
		}
	}

	public final int getTextureHandle() {
		return this.textureHandle[0];
	}

	public void deleteTexture() {
		if (this.textureHandle[0] > 0) {
			GLES20.glDeleteTextures(1, this.textureHandle, 0);
			this.textureHandle[0] = -1;
		}
	}

	public static int getGLTexNum(int texNum) {
		int ret = GLES20.GL_TEXTURE0;
		switch (texNum) {
		case 0:
			ret = GLES20.GL_TEXTURE0;
			break;
		case 1:
			ret = GLES20.GL_TEXTURE1;
			break;
		case 2:
			ret = GLES20.GL_TEXTURE2;
			break;
		case 3:
			ret = GLES20.GL_TEXTURE3;
			break;
		case 4:
			ret = GLES20.GL_TEXTURE4;
			break;
		case 5:
			ret = GLES20.GL_TEXTURE5;
			break;
		case 6:
			ret = GLES20.GL_TEXTURE6;
			break;
		case 7:
			ret = GLES20.GL_TEXTURE7;
			break;
		case 8:
			ret = GLES20.GL_TEXTURE8;
			break;
		case 9:
			ret = GLES20.GL_TEXTURE9;
			break;
		case 10:
			ret = GLES20.GL_TEXTURE10;
			break;
		case 11:
			ret = GLES20.GL_TEXTURE11;
			break;
		case 12:
			ret = GLES20.GL_TEXTURE12;
			break;
		case 13:
			ret = GLES20.GL_TEXTURE13;
			break;
		case 14:
			ret = GLES20.GL_TEXTURE14;
			break;
		case 15:
			ret = GLES20.GL_TEXTURE15;
			break;
		default:
			break;
		}
		return ret;
	}

	public static int getMaxCombinedTextures() {
		if (maxCombinedTextures == -1) {
			final int[] ret = new int[1];
			GLES20.glGetIntegerv(GLES20.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, ret, 0);
			maxCombinedTextures = ret[0];
		}
		return maxCombinedTextures;
	}
}
