package com.laguerre.mytest;

import java.util.Arrays;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import com.laguerre.kan.Camera;
import com.laguerre.kan.Color;
import com.laguerre.kan.Light;
import com.laguerre.kan.Lights;
import com.laguerre.kan.Material;
import com.laguerre.kan.MeshRenderer;
import com.laguerre.kan.PassShader;
import com.laguerre.kan.ImageEffects;
import com.laguerre.kan.Quaternion;
import com.laguerre.kan.RenderTexture;
import com.laguerre.kan.RenderingSettings;
import com.laguerre.kan.SceneConductor;
import com.laguerre.kan.SceneObject;
import com.laguerre.kan.Shader;
import com.laguerre.kan.SurfaceInfo;
import com.laguerre.kan.Texture2D;
import com.laguerre.kan.Utils;
import com.laguerre.kan.Vector3;

import android.content.Context;
import android.opengl.GLSurfaceView;

public class MyGL20Renderer implements GLSurfaceView.Renderer {
	private Context context;

	public MyGL20Renderer(Context context) {
		this.context = context;
		// To be accessible later, somewhere else...
		Utils.setContext(context);
	}

	@Override
    public void onSurfaceCreated(GL10 unused, EGLConfig config) {
		SurfaceInfo.resetInstance();
		RenderingSettings.resetInstance();
		Lights.resetInstance();
		SceneConductor.resetInstance();
		ImageEffects.resetInstance();

		buildWorld();

		/*
		TestCamera.reset();
		TestFBO.reset();

		TestCamera.getInstance();
		TestFBO.getInstance();

		TestCamera.getInstance().init();
		TestFBO.getInstance().init();
*/

	}

	private void buildWorld() {
		ImageEffects.getInstance().init();

		RenderingSettings.getInstance().setAmbientLight(new Color(0.1f, 0.1f, 0.1f, 1f));

		final SceneObject myCamera = SceneConductor.getInstance().addSceneObject("Camera");
		myCamera.getSharedTransform().translateGlobally(0f, 0f, 5f);
		myCamera.addComponent(Camera.class);
		//myCamera.addComponent(PerlinImageEffect.class);
		//myCamera.addComponent(GaussianBlurImageEffect.class);
		//myCamera.addComponent(LaplaceEdgeImageEffect.class);
		myCamera.getSharedCamera().setAsCurrent();

		//addTriangle();
		//addQuad();
		addCube();

		final SceneObject lightSob0 = SceneConductor.getInstance().addSceneObject("Light 0");
		lightSob0.getSharedTransform().translateGlobally(0.5f, 0.2f, 1f);
		final Light light0 = lightSob0.addComponent(Light.class);
		light0.setLightType(Light.SPOT);
		light0.setDiffuseColor(0.5f, 0.5f, 0.4f, 0.7f);
		light0.setSpecularColor(0.8f, 0.8f, 0.75f, 0.8f);
		light0.setInnerCutOffAngle(2f);
		light0.setOuterCutOffAngle(10f);

		final SceneObject lightSob1 = SceneConductor.getInstance().addSceneObject("Light 1");
		lightSob1.getSharedTransform().translateGlobally(0f, 0.4f, -0.5f);
		Quaternion q = new Quaternion(-90f, Vector3.UP);
		q.mulSelf(new Quaternion(-30f, Vector3.RIGHT));
		lightSob1.getSharedTransform().rotateGlobally(q);
		final Light light1 = lightSob1.addComponent(Light.class);
		light1.setLightType(Light.DIRECTIONAL);
		light1.setDiffuseColor(0.8f, 0.8f, 0.5f, 1f);
		light1.setSpecularColor(1.0f, 1.0f, 0.7f, 1f);

//		final Texture2D tex = new Texture2D();
//		tex.setBitmap(this.context, R.drawable.carte_mcdo);
//		tex.create();
//		sobMatsetShaderTexture("texture0", tex);

		/*
		final Camera camToTex = sob.addComponent(Camera.class, "Cam to texture");
		final RenderTexture renderTex = new RenderTexture();
		renderTex.init(256, 256);
		renderTex.create();
		camToTex.setRenderTexture(renderTex);
		sobMat.setShaderTexture("texture0", renderTex);
		SceneConductor.getInstance().addCamera(camToTex);
		*/
	}

	private void addTriangle() {
		final SceneObject sob = SceneConductor.getInstance().addSceneObject("Triangle");
		sob.getSharedTransform().translateGlobally(0.2f, 0f, -1f);

		final SimpleTriangle mesh = sob.addComponent(SimpleTriangle.class, "Simple triangle");
		MeshRenderer sobRenderer = sob.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material(0);
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		/*
		final PassShader passShader1 = shader.addPassShader(1);
		passShader1.useExistingProgram(passShader0.getProgramHandle());
		*/

		sobRenderer.addMaterial(sobMat);

		mesh.init();
		mesh.build();
	}

	private void addQuad() {
		final SceneObject sob = SceneConductor.getInstance().addSceneObject("Quad");
		sob.getSharedTransform().translateGlobally(-0.2f, 0f, -1f);

		final SimpleQuad mesh = sob.addComponent(SimpleQuad.class, "Simple quad");
		MeshRenderer sobRenderer = sob.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material(0);
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		/*
		final PassShader passShader1 = shader.addPassShader(1);
		passShader1.useExistingProgram(passShader0.getProgramHandle());
		*/

		sobRenderer.addMaterial(sobMat);

		mesh.init();
		mesh.build();
	}

	private void addCube() {
		final SceneObject sobParent = SceneConductor.getInstance().addSceneObject("Cube parent");
		sobParent.getSharedTransform().translateGlobally(0.5f, 0.2f, -1.5f);

		final SceneObject sobCube = SceneConductor.getInstance().addSceneObject("Cube");
		sobCube.getSharedTransform().setParent(sobParent.getSharedTransform());

		final Quaternion q = new Quaternion(30f, Vector3.RIGHT);
		q.mulSelf(new Quaternion(15f, Vector3.UP));
		sobCube.getSharedTransform().rotateLocally(q);

		final Cube mesh = sobCube.addComponent(Cube.class, "Cube");
		MeshRenderer sobRenderer = sobCube.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material(0);
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		/*
		final PassShader passShader1 = shader.addPassShader(1);
		passShader1.useExistingProgram(passShader0.getProgramHandle());
		*/

		sobRenderer.addMaterial(sobMat);

		mesh.init();
		mesh.build();
	}

	@Override
    public void onDrawFrame(GL10 unused) {
		if (initDone) {
	        SceneConductor.getInstance().drawFrame();

	        /*
			TestCamera.getInstance().renderToTexture();
			//TestFBO.getInstance().renderToTexture();
			TestFBO.getInstance().renderToQuad();
			*/
		}
    }


	private boolean initDone = false;

	@Override
    public void onSurfaceChanged(GL10 unused, int width, int height) {
		width = 800;
		height = 1108;

    	if (initDone) return;
    	initDone = true;

		SurfaceInfo.getInstance().setSurfaceSize(width, height);

		ImageEffects.getInstance().setRenderQuadDim(width, height);

		Camera.getCurrent().setRenderTexture(
				ImageEffects.getInstance().getSharedRenderTexture());

		SceneConductor.getInstance().onRenderSurfaceChanged(width, height);

/*
    	TestFBO.getInstance().setDim(width, height);
    	TestCamera.getInstance().setCameraRenderTexture(
    			TestFBO.getInstance().getSharedRenderTexture());
*/
    }
}
